#include <utility>

#ifndef QR_INVOICE_PAYMENT_PART_RECEIPT_CREATOR_HPP
#define QR_INVOICE_PAYMENT_PART_RECEIPT_CREATOR_HPP

#include <qrinvoice/qrinvoicedefs.hpp>
#include <qrinvoice/output/payment_part_receipt.hpp>
#include <qrinvoice/model/qr_invoice.hpp>
#include <qrinvoice/output_format.hpp>
#include <qrinvoice/output_resolution.hpp>
#include <qrinvoice/page_size.hpp>
#include <qrinvoice/locale.hpp>


namespace qrinvoice {


class payment_part_receipt_creator {
public:
	payment_part_receipt_creator() = default;
	payment_part_receipt_creator(model::qr_invoice qr_invoice, page_size pg_sz,
			output_format fmt, output_resolution res, qrinvoice::locale lcl, bool boundary, bool scissors, bool separation_text)
		:qr_invoice_{std::move(qr_invoice)}
		,page_size_{ pg_sz }
		,output_format_{ fmt }
		,output_resolution_{ res }
		,locale_{ lcl }
		,boundary_lines_{ boundary }
		,boundary_line_scissors_ { scissors }
		,boundary_line_separation_text_{ separation_text } {}

public:
	payment_part_receipt_creator& qr_invoice(const model::qr_invoice&  qr_invoice) {
		qr_invoice_ = qr_invoice;
		return *this;
	}

	payment_part_receipt_creator& page_size(const page_size& page_size) {
		page_size_ = page_size;
		return *this;
	}

	payment_part_receipt_creator& output_format(const output_format output_format) {
		output_format_ = output_format;
		return *this;
	}

	payment_part_receipt_creator& output_resolution(const output_resolution output_resolution) {
		output_resolution_ = output_resolution;
		return *this;
	}

	payment_part_receipt_creator& locale(qrinvoice::locale locale) {
		locale_ = locale;
		return *this;
	}

	payment_part_receipt_creator& boundary_lines(bool boundary_lines) {
		boundary_lines_ = boundary_lines;
		return *this;
	}

	payment_part_receipt_creator& with_boundary_lines() {
		return boundary_lines(true);
	}

	payment_part_receipt_creator& without_boundary_lines() {
		return boundary_lines(true);
	}

	payment_part_receipt_creator& scissors(bool boundary_line_scissors) {
		boundary_line_scissors_ = boundary_line_scissors;
		return *this;
	}
	payment_part_receipt_creator& with_scissors() {
		return scissors(true);
	}

	payment_part_receipt_creator& without_scissors() {
		return scissors(true);
	}

	payment_part_receipt_creator& separation_text(bool boundary_line_separation_text) {
		boundary_line_separation_text_ = boundary_line_separation_text;
		return *this;
	}

	payment_part_receipt_creator& with_separation_text() {
		return separation_text(true);
	}

	payment_part_receipt_creator& without_separation_text() {
		return separation_text(true);
	}

	payment_part_receipt_creator& in_german() {
		locale_ = qrinvoice::locale::german;
		return *this;
	}

	payment_part_receipt_creator& in_french() {
		locale_ = qrinvoice::locale::french;
		return *this;
	}

	payment_part_receipt_creator& in_english() {
		locale_ = qrinvoice::locale::english;
		return *this;
	}

	payment_part_receipt_creator& in_italian() {
		locale_ = qrinvoice::locale::italian;
		return *this;
	}

	QRINVOICE_API output::payment_part_receipt create_payment_part_receipt() const;

private:
	model::qr_invoice			qr_invoice_;
	qrinvoice::page_size			page_size_ = qrinvoice::page_size::a4;
	qrinvoice::output_format		output_format_ = qrinvoice::output_format::pdf;
	qrinvoice::output_resolution		output_resolution_ = qrinvoice::output_resolution::medium_300_dpi;
	qrinvoice::locale			locale_ = qrinvoice::locale::german;
	bool					boundary_lines_ = true;
	bool					boundary_line_scissors_ = true;
	bool					boundary_line_separation_text_ = true;
};

} // namespace qrinvoice


#endif // QR_INVOICE_PAYMENT_PART_RECEIPT_CREATOR_HPP

